@@ -1,15 +1,18 @@
 #include <igraph.h>
 
-int main() {
+int main(void) {
+  igraph_integer_t num_vertices = 1000;
+  igraph_integer_t num_edges = 1000;
   igraph_real_t diameter;
   igraph_t graph;
 
   igraph_rng_seed(igraph_rng_default(), 42);
 
-  igraph_erdos_renyi_game(&graph, IGRAPH_ERDOS_RENYI_GNM, 1000, 3000,
+  igraph_erdos_renyi_game(&graph, IGRAPH_ERDOS_RENYI_GNM,
+                          num_vertices, num_edges,
                           IGRAPH_UNDIRECTED, IGRAPH_NO_LOOPS);
 
-  igraph_diameter(&graph, &diameter, 0, 0, 0, IGRAPH_UNDIRECTED, 1);
+  igraph_diameter(&graph, &diameter, NULL, NULL, NULL, NULL, IGRAPH_UNDIRECTED, /* unconn= */ true);
   printf("Diameter of a random graph with average degree %g: %g\n",
           2.0 * igraph_ecount(&graph) / igraph_vcount(&graph),
           (double) diameter);
