@@ -74,12 +74,14 @@
    /* Put the tokens into the symbol table, so that GDB and other debuggers
       know about them.  */
    enum yytokentype {
+     END = 0,
      ALNUM = 258,
      NEWLINE = 259,
      ERROR = 260
    };
 #endif
 /* Tokens.  */
+#define END 0
 #define ALNUM 258
 #define NEWLINE 259
 #define ERROR 260
@@ -113,24 +115,22 @@
 
 */
 
-#include <stdio.h>
-#include <string.h>
-
 #include "igraph_types.h"
 #include "igraph_memory.h"
 #include "igraph_error.h"
-#include "config.h"
 
-#include "core/math.h"
 #include "io/ncol-header.h"
 #include "io/parsers/ncol-parser.h"
 #include "io/parsers/ncol-lexer.h"
+#include "io/parse_utils.h"
 #include "internal/hacks.h"
 
+#include <stdio.h>
+#include <string.h>
+
 int igraph_ncol_yyerror(YYLTYPE* locp,
                         igraph_i_ncol_parsedata_t *context,
                         const char *s);
-igraph_real_t igraph_ncol_get_number(const char *str, long int len);
 
 #define scanner context->scanner
 
@@ -157,8 +157,8 @@ igraph_real_t igraph_ncol_get_number(const char *str, long int len);
 typedef union YYSTYPE
 
 {
-  long int edgenum;
-  double weightnum;
+  igraph_integer_t edgenum;
+  igraph_real_t weightnum;
 }
 /* Line 193 of yacc.c.  */
 
@@ -472,7 +472,7 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,    93,    93,    94,    95,    98,   103,   111,   116
+       0,    92,    92,    93,    94,    97,   102,   110,   120
 };
 #endif
 
@@ -481,8 +481,9 @@ static const yytype_uint8 yyrline[] =
    First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
 static const char *const yytname[] =
 {
-  "$end", "error", "$undefined", "ALNUM", "NEWLINE", "ERROR", "$accept",
-  "input", "edge", "edgeid", "weight", 0
+  "\"end of file\"", "error", "$undefined", "\"alphanumeric\"",
+  "\"end of line\"", "ERROR", "$accept", "input", "edge", "edgeid",
+  "weight", 0
 };
 #endif
 
@@ -1404,34 +1405,44 @@ yyreduce:
         case 5:
 
     {
-           igraph_vector_push_back(context->vector, (yyvsp[(1) - (3)].edgenum));
-           igraph_vector_push_back(context->vector, (yyvsp[(2) - (3)].edgenum));
-           igraph_vector_push_back(context->weights, 0);
+           IGRAPH_YY_CHECK(igraph_vector_int_push_back(context->vector, (yyvsp[(1) - (3)].edgenum)));
+           IGRAPH_YY_CHECK(igraph_vector_int_push_back(context->vector, (yyvsp[(2) - (3)].edgenum)));
+           IGRAPH_YY_CHECK(igraph_vector_push_back(context->weights, 0.0));
        ;}
     break;
 
   case 6:
 
     {
-           igraph_vector_push_back(context->vector, (yyvsp[(1) - (4)].edgenum));
-           igraph_vector_push_back(context->vector, (yyvsp[(2) - (4)].edgenum));
-           igraph_vector_push_back(context->weights, (yyvsp[(3) - (4)].weightnum));
+           IGRAPH_YY_CHECK(igraph_vector_int_push_back(context->vector, (yyvsp[(1) - (4)].edgenum)));
+           IGRAPH_YY_CHECK(igraph_vector_int_push_back(context->vector, (yyvsp[(2) - (4)].edgenum)));
+           IGRAPH_YY_CHECK(igraph_vector_push_back(context->weights, (yyvsp[(3) - (4)].weightnum)));
            context->has_weights = 1;
        ;}
     break;
 
   case 7:
 
-    { igraph_trie_get2(context->trie,
-                  igraph_ncol_yyget_text(scanner),
-                  igraph_ncol_yyget_leng(scanner),
-                  &(yyval.edgenum)); ;}
+    {
+  igraph_integer_t trie_id;
+  IGRAPH_YY_CHECK(igraph_trie_get_len(context->trie,
+    igraph_ncol_yyget_text(scanner),
+    igraph_ncol_yyget_leng(scanner),
+    &trie_id
+  ));
+  (yyval.edgenum) = trie_id;
+;}
     break;
 
   case 8:
 
-    { (yyval.weightnum)=igraph_ncol_get_number(igraph_ncol_yyget_text(scanner),
-                        igraph_ncol_yyget_leng(scanner)); ;}
+    {
+    igraph_real_t val;
+    IGRAPH_YY_CHECK(igraph_i_parse_real(igraph_ncol_yyget_text(scanner),
+                                        igraph_ncol_yyget_leng(scanner),
+                                        &val));
+    (yyval.weightnum)=val;
+;}
     break;
 
 
@@ -1668,14 +1679,3 @@ int igraph_ncol_yyerror(YYLTYPE* locp,
     return 0;
 }
 
-igraph_real_t igraph_ncol_get_number(const char *str, long int length) {
-    igraph_real_t num;
-    char *tmp=IGRAPH_CALLOC(length+1, char);
-
-    strncpy(tmp, str, length);
-    tmp[length]='\0';
-    sscanf(tmp, "%lf", &num);
-    IGRAPH_FREE(tmp);
-    return num;
-}
-
