@@ -46,49 +46,50 @@
      QSTR = 261,
      PSTR = 262,
      NETWORKLINE = 263,
-     VERTICESLINE = 264,
-     ARCSLINE = 265,
-     EDGESLINE = 266,
-     ARCSLISTLINE = 267,
-     EDGESLISTLINE = 268,
-     MATRIXLINE = 269,
-     ERROR = 270,
-     VP_X_FACT = 271,
-     VP_Y_FACT = 272,
-     VP_IC = 273,
-     VP_BC = 274,
-     VP_LC = 275,
-     VP_LR = 276,
-     VP_LPHI = 277,
-     VP_BW = 278,
-     VP_FOS = 279,
-     VP_PHI = 280,
-     VP_R = 281,
-     VP_Q = 282,
-     VP_LA = 283,
-     VP_FONT = 284,
-     VP_URL = 285,
-     VP_SIZE = 286,
-     EP_C = 287,
-     EP_S = 288,
-     EP_A = 289,
-     EP_W = 290,
-     EP_H1 = 291,
-     EP_H2 = 292,
-     EP_A1 = 293,
-     EP_A2 = 294,
-     EP_K1 = 295,
-     EP_K2 = 296,
-     EP_AP = 297,
-     EP_P = 298,
-     EP_L = 299,
-     EP_LP = 300,
-     EP_LR = 301,
-     EP_LPHI = 302,
-     EP_LC = 303,
-     EP_LA = 304,
-     EP_SIZE = 305,
-     EP_FOS = 306
+     NET_TITLE = 264,
+     VERTICESLINE = 265,
+     ARCSLINE = 266,
+     EDGESLINE = 267,
+     ARCSLISTLINE = 268,
+     EDGESLISTLINE = 269,
+     MATRIXLINE = 270,
+     ERROR = 271,
+     VP_X_FACT = 272,
+     VP_Y_FACT = 273,
+     VP_IC = 274,
+     VP_BC = 275,
+     VP_LC = 276,
+     VP_LR = 277,
+     VP_LPHI = 278,
+     VP_BW = 279,
+     VP_FOS = 280,
+     VP_PHI = 281,
+     VP_R = 282,
+     VP_Q = 283,
+     VP_LA = 284,
+     VP_FONT = 285,
+     VP_URL = 286,
+     VP_SIZE = 287,
+     EP_C = 288,
+     EP_S = 289,
+     EP_A = 290,
+     EP_W = 291,
+     EP_H1 = 292,
+     EP_H2 = 293,
+     EP_A1 = 294,
+     EP_A2 = 295,
+     EP_K1 = 296,
+     EP_K2 = 297,
+     EP_AP = 298,
+     EP_P = 299,
+     EP_L = 300,
+     EP_LP = 301,
+     EP_LR = 302,
+     EP_LPHI = 303,
+     EP_LC = 304,
+     EP_LA = 305,
+     EP_SIZE = 306,
+     EP_FOS = 307
    };
 #endif
 /* Tokens.  */
@@ -99,49 +100,50 @@
 #define QSTR 261
 #define PSTR 262
 #define NETWORKLINE 263
-#define VERTICESLINE 264
-#define ARCSLINE 265
-#define EDGESLINE 266
-#define ARCSLISTLINE 267
-#define EDGESLISTLINE 268
-#define MATRIXLINE 269
-#define ERROR 270
-#define VP_X_FACT 271
-#define VP_Y_FACT 272
-#define VP_IC 273
-#define VP_BC 274
-#define VP_LC 275
-#define VP_LR 276
-#define VP_LPHI 277
-#define VP_BW 278
-#define VP_FOS 279
-#define VP_PHI 280
-#define VP_R 281
-#define VP_Q 282
-#define VP_LA 283
-#define VP_FONT 284
-#define VP_URL 285
-#define VP_SIZE 286
-#define EP_C 287
-#define EP_S 288
-#define EP_A 289
-#define EP_W 290
-#define EP_H1 291
-#define EP_H2 292
-#define EP_A1 293
-#define EP_A2 294
-#define EP_K1 295
-#define EP_K2 296
-#define EP_AP 297
-#define EP_P 298
-#define EP_L 299
-#define EP_LP 300
-#define EP_LR 301
-#define EP_LPHI 302
-#define EP_LC 303
-#define EP_LA 304
-#define EP_SIZE 305
-#define EP_FOS 306
+#define NET_TITLE 264
+#define VERTICESLINE 265
+#define ARCSLINE 266
+#define EDGESLINE 267
+#define ARCSLISTLINE 268
+#define EDGESLISTLINE 269
+#define MATRIXLINE 270
+#define ERROR 271
+#define VP_X_FACT 272
+#define VP_Y_FACT 273
+#define VP_IC 274
+#define VP_BC 275
+#define VP_LC 276
+#define VP_LR 277
+#define VP_LPHI 278
+#define VP_BW 279
+#define VP_FOS 280
+#define VP_PHI 281
+#define VP_R 282
+#define VP_Q 283
+#define VP_LA 284
+#define VP_FONT 285
+#define VP_URL 286
+#define VP_SIZE 287
+#define EP_C 288
+#define EP_S 289
+#define EP_A 290
+#define EP_W 291
+#define EP_H1 292
+#define EP_H2 293
+#define EP_A1 294
+#define EP_A2 295
+#define EP_K1 296
+#define EP_K2 297
+#define EP_AP 298
+#define EP_P 299
+#define EP_L 300
+#define EP_LP 301
+#define EP_LR 302
+#define EP_LPHI 303
+#define EP_LC 304
+#define EP_LA 305
+#define EP_SIZE 306
+#define EP_FOS 307
 
 
 
@@ -150,11 +152,11 @@
 typedef union YYSTYPE
 
 {
-  long int intnum;
-  double   realnum;
+  igraph_integer_t intnum;
+  igraph_real_t    realnum;
   struct {
     char *str;
-    int len;
+    size_t len;
   } string;
 }
 /* Line 1529 of yacc.c.  */
