@@ -26,6 +26,7 @@
 
 #include "igraph_decls.h"
 #include "igraph_datatype.h"
+#include "igraph_error.h"
 #include "igraph_vector.h"
 #include "igraph_vector_ptr.h"
 
@@ -35,11 +36,9 @@ __BEGIN_DECLS
 #define IGRAPH_MERGE_MODE_INTERSECTION 2
 
 int igraph_i_order_edgelist_cmp(void *edges, const void *e1, const void *e2);
-int igraph_i_merge(igraph_t *res, int mode,
+igraph_error_t igraph_i_merge(igraph_t *res, int mode,
                    const igraph_t *left, const igraph_t *right,
-                   igraph_vector_t *edge_map1, igraph_vector_t *edge_map2);
-void igraph_i_union_intersection_destroy_vectors(igraph_vector_ptr_t *v);
-void igraph_i_union_intersection_destroy_vector_longs(igraph_vector_ptr_t *v);
+                   igraph_vector_int_t *edge_map1, igraph_vector_int_t *edge_map2);
 
 __END_DECLS
 
