@@ -1,6 +1,6 @@
 #include <igraph.h>
 
-#include "../unit/test_utilities.inc"
+#include "../unit/test_utilities.h"
 
 /* Regression test for https://github.com/igraph/igraph/issues/1814 */
 
@@ -26,14 +26,14 @@ void test_igraph_to_undirected(igraph_to_undirected_t mode) {
     igraph_to_undirected(&g, mode, &comb);
     igraph_attribute_combination_destroy(&comb);
 
-    igraph_write_graph_gml(&g, stdout, 0, "unittest");
+    igraph_write_graph_gml(&g, stdout, IGRAPH_WRITE_GML_DEFAULT_SW, 0, "");
 
     igraph_destroy(&g);
 
     VERIFY_FINALLY_STACK();
 }
 
-int main() {
+int main(void) {
     igraph_set_attribute_table(&igraph_cattribute_table);
 
     printf("to_undirected(COLLAPSE)\n");
