@@ -25,11 +25,11 @@
 
 #include "core/cutheap.h"
 
-#include "test_utilities.inc"
+#include "test_utilities.h"
 
-int main() {
+int main(void) {
     igraph_i_cutheap_t ch;
-    long int i;
+    igraph_integer_t i;
 
     igraph_i_cutheap_init(&ch, 10);
 
@@ -37,8 +37,8 @@ int main() {
         igraph_i_cutheap_update(&ch, i, i);
     }
     while (!igraph_i_cutheap_empty(&ch)) {
-        long int idx = igraph_i_cutheap_popmax(&ch);
-        printf("%li ", idx);
+        igraph_integer_t idx = igraph_i_cutheap_popmax(&ch);
+        printf("%" IGRAPH_PRId " ", idx);
     }
     printf("\n");
 
